function getwholebrainconnectivitywithseed (seedts, imgdir,maskname,scan,tr,justeventmeas,scanofinterest,lambdatouse,zdim,xdim,ydim)
% given a seed timeseries and a directory of images, writes volumes
% for each type of connectivity across the whole brain in the
% current directory
% scan is the number of scans in each trial in a slow-event-
%   related design (or per block, in a block design where connectivity 
%   within blocks is of interest)
% tr is the number of seconds between samples (scans)
% justeventmeas indicates that only event-related measures
%   should be calculated (e.g., when only certain trials
%   are being passed to the routine
% scanofinterest is a specific scan number within trials 
%   at which something interesting is hypothesized to 
%   occur (e.g., the peak of the BOLD response)

if nargin<11, ydim=64; end
if nargin<10, xdim=64; end
if nargin<9, zdim=34; end
if nargin<8, lamdatouse=1; end

%addpath('/data/Siegle/matlab/fmritools');
%addpath('/data/Siegle/matlab/connectivitytoolbox');

imglist=dir(sprintf('%s/*.img',imgdir));

ts=zeros(length(imglist),xdim,ydim,zdim);
for ct=1:length(imglist)
  ts(ct,:,:,:)=readvolume(char(imglist(ct).name),zdim,xdim,ydim,1);
  fprintf('.');
end
fprintf('\n');

if maskname
  mask=readvolume(maskname,zdim,xdim,ydim);
else
  mask=ones(xdim,ydim,zdim);
end
goodvox=find(mask);

if lambdatouse==0
% get the lambda from 10 random voxels
 randvox=round(rand(1,10).*length(goodvox));
 for ct=1:10
   [cr,u,v,lambda(ct)]=ccorr_cv(seedts,ts(:,goodvox(randvox(ct))),scan);
   fprintf('f');
 end
 fprintf('\n');
 lambdatouse=mean(lambda);
 if lambdatouse==0
   fprintf('WARNING: Lambda is zero... estimating per voxel - this will be slow\n');
 end
end

pcorr=zeros(xdim,ydim,zdim); scancorr=zeros(xdim,ydim,zdim); ccorr=zeros(xdim,ydim,zdim);
for x=1:xdim
  for y=1:ydim
    for z=1:zdim
      if mask(x,y,z)
	s=getallconnectivityinds(seedts,squeeze(ts(:,x,y,z)),scan,tr,justeventmeas,scanofinterest,lambdatouse);
	pcorr(x,y,z)=s.pcorr;
	scancorr(x,y,z)=s.scancorr;
	ccorr(x,y,z)=s.ccorr;
      end
    end
    fprintf('.');
  end
  fprintf('\n');
end
writevolume('pcorr.img',pcorr.*100,1);
writevolume('scancorr.img',scancorr.*100,1);
writevolume('ccorr.img',ccorr.*100,1);
